
/* Code is based on the library of Gijs Withagen (mainly for microstepping and changed prescaler).
 * Comment from Gijs:
 *    Use 3 PWM pins to drive motor
 *    Choose 9,10,11 and 6,5,3
 *    Had to change the prescaler wit setPwmFrequency, @TODO need to clean
 *    this up. And have a challenge to get a repaired micros() and millis().
 *    because I would like to use PJON for a 1 wire communication bus.

 Comments Olivier
 *  USE Arduino IDE Version 1.8.7 which has modified wiring.c in the core!

************************************************************
*LIBRARY MotorVID28_05 Definitions:
 VARIABLES TO USED AS INPUT:
 - currentPosition (signed int) - current position of the handle
 - maxSpeed (signed int)        - speed after acceleration and before deceleration. Every X tick a step. Ticks are all 600ms
 - accel (signed int)           - 0=no acceleration 1=4.2° for acceleration (and same for decceleration), 2=8.4° etc.
 - waitDelay (unsigned int)     - how many ms shall the handle wait after the cycle is started to start running. It's used to let some handles start later than others
 VARIABLES TO USED AS READ ONLY!:
 - destination (signed int)     - The destination as absolute value. 1 turn is 4320 steps
 - forward (bool)               - if true, then it runs forward, if false then backwards
 - distance (unsigned int)      - Distance between handle position and destination
 - posReached(bool)             - true if new position is reached. Is set to false when new destination is saved with procedure "setDestination()"
 PROCEDURES:
 setDestination(signed int)     - Input the target destination in absolute value
 startNewMove()                 - Only when this procedure is called, then the timers start running (waitDelay starts, or the handles start accelerating)
 run()                          - call as often as possible. It checks if the time is ready for a next tick. if yes, it checks if it's time to run a handle
 stepUp()                       - call to manually make a step forward
 stepDown()                     - call to manually make a step backward

 Comments to Library:
 - Tick every 600microseconds. ATMEGA is able to handle both handles at 8MHz and 600us. With 16MHz it should be possible with 0.5ms(500us)
 - good combination 2 handle max_speed / self_a_ticks_winkel (s/full turn) = 4/7(~7), 2/(~)
 - Library defines the motor pins as output pins.
 
************************************************************

VERSIONS
V00
 - Implementation of new library to zeroing the handles with Hall sensor, running to position 0 and then to a new position

************************************************************
Bugs:
 - Serial is not working!!! remove all Serial communication!
 - Hardware: solved on one print: A2 und A3 are floating, need a pull down resistor!
 - Hardware: Change print with a resistor to common of I2C selection Switch
 - Nullen: Exit when no hall signal is found after 2 turns. Error message!
 - Can resolution be changed to 500 micro (in library) with 16MHz crystal? Would make the calculation of times easier (amount of steps, * speed (p.ex3) * 0.5 ms = time im milliseconds

 */

// ---------- INCLUDE LIBRARIES --------------------
#include <Arduino.h>
#include <MotorVID28_05.h> // modified library from Gijs Withagen
#include <Wire.h> //I2C Library

// ------------- OLED initialization (can be removed later) ------------------
#include <SPI.h> // OLED
#include <Adafruit_GFX.h> //OLED
#include <Adafruit_SSD1306.h> // OLED
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels
#define OLED_RESET     4 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

// ------------- CREATE STEPPER OBJECT ---------------
const int Mot1Pin1 = 9;
const int Mot1Pin2 = 10;
const int Mot1Pin3 = 11;
const int Mot2Pin1 = 3;
const int Mot2Pin2 = 5;
const int Mot2Pin3 = 6;
MotorVID28_05 motor1(360 * 3 * 4,  Mot1Pin1, Mot1Pin2, Mot1Pin3); 
MotorVID28_05 motor2(360 * 3 * 4,  Mot2Pin1, Mot2Pin2, Mot2Pin3); 



// ------------ DEFINE VARIABLES ---------------------
byte HallPin1=A0;   // Hall sensor 1
byte HallPin2=A1;   // Hall sensor 2
bool  M1initialized = false; // if false, the handle is not homed with Hall sensor 
bool  M2initialized = false; 
int   offsetHand1 = 1506; // Defines the position of the Hall sensor relative to position zero
int   offsetHand2 = 1506; // 1503 ok 7iel 1550 Defines the position of the Hall sensor relative to position zero


// ----------------------------------------------------------------------------
// ------------------------------  SETUP --------------------------------------
// ----------------------------------------------------------------------------
void setup() {

  pinMode(HallPin1, INPUT); // Hall sensor 1
  pinMode(HallPin2, INPUT); // Hall sensor 2

  // ---------- SET PRESCALER -----------------
  // Change timing of MCU (done by Gijs Withager). Arduino IDE version 1.8.7 with modified wire.c required for this
  setPrescaler(0, 1);
  setPrescaler(1, 1);
  setPrescaler(2, 1);


  // ---- initialize OLED ----
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3C for 128x32
    for(;;); // Don't proceed, loop forever
  }

  // ----------------- INITIALIZE MOTORS ----------------------
  Zeroing(); // initialize both handles
  
  motor1.setDestination(2160); 
  motor1.maxSpeed=3;
  motor1.accel=5;
  motor2.setDestination(-2160);
  motor2.maxSpeed=3;
  motor2.accel=5;
  PrintDisplayText(0,"  GEHT LOS");
  delay(1000);
  motor1.startNewMove();
  motor2.startNewMove();
  PrintDisplayText(0,"  NEW MOVE");
/*
  display.clearDisplay();
  display.setCursor(0,0);             // (X,Y)
  display.println("BUG"); //6
  display.setCursor(50,0);             
  display.println(motor2.debugg);
  display.setCursor(0,10);             
  display.println("DEST");
  display.setCursor(50,10);            
  display.println(motor2.destination); //2160
  display.setCursor(0,20);
  display.println("DIST");
  display.setCursor(50,20);            
  display.println(motor2.distance); //2160
  display.display();
*/
}


// ----------------------------------------------------------------------------
// ------------------------------  LOOP ---------------------------------------
// ----------------------------------------------------------------------------
void loop(void)
{
  
  motor1.run();
  motor2.run();
/*    display.clearDisplay();
  display.setCursor(0,0);             // Start at top-left corner
  display.println("BUG A"); //6
  display.setCursor(50,0);             // Start at top-left corner
  display.println(motor2.debugg);
  display.setCursor(0,10);             // Start at top-left corner
  display.println("POS 1");
  display.setCursor(50,10);             // Start at top-left corner
  display.println(motor2.currentPosition);
  display.setCursor(0,20);             // Start at top-left corner
  display.println("MOVE A"); //-1506
  display.setCursor(50,20);             // Start at top-left corner
  display.println(motor2.moved);
  display.display();*/

//  PrintDisplayValue(10,motor2.currentPosition);
  if(motor1.posReached==true && motor2.posReached==true){
    PrintDisplayText(10,"   ZIEL ERREICHT");
    newPos();
    }
  
}




// ----------------------------------------------------------------------------
// ------------------------------  ZEROING  -----------------------------------
// ----------------------------------------------------------------------------
void Zeroing(void){ 
  int LowestHall1Val = 0; // Reset Lowest Hall measurement
  int LowestHall2Val = 0; // Reset Lowest Hall measurement
  int hallVal =0;//actual read of Hall sensor
  
  do{
    if(M1initialized==false){ // look for Hall sensor for motor 1
      motor1.stepUp();
      hallVal=analogRead(HallPin1);
      if (hallVal > LowestHall1Val) {LowestHall1Val = hallVal;}      
      if (hallVal < (LowestHall1Val - 10)) {M1initialized=true;  PrintDisplayText(0,"HAND 1 OK");}
    }
    if(M2initialized==false){ // look for Hall sensor for motor 2
      motor2.stepUp();
      hallVal=analogRead(HallPin2);
      if (hallVal > LowestHall2Val) {LowestHall2Val = hallVal;}      
      if (hallVal < (LowestHall2Val - 10)) {M2initialized=true;  PrintDisplayText(0,"HAND 2 OK");}
    }
    delay(2);
  } while(M1initialized==false || M2initialized==false); // repeat the search for hall sensor until both handles found their position
  motor1.currentPosition = offsetHand1; // set the actual position of the handle
  motor2.currentPosition = offsetHand2; // set the actual position of the handle

  // run the handles to zero position
  motor1.setDestination(0);
  motor1.maxSpeed=3;
  motor1.accel=5;
  motor2.setDestination(0);
  motor2.maxSpeed=3;
  motor2.accel=5;
/*  display.clearDisplay();
  display.setCursor(0,0);             // Start at top-left corner
  display.println("BUG B");
  display.setCursor(50,0);             // Start at top-left corner
  display.println(motor2.debugg);
  display.setCursor(0,10);             // Start at top-left corner
  display.println("POS 1");
  display.setCursor(50,10);             // Start at top-left corner
  display.println(motor2.currentPosition);
  display.setCursor(0,20);             // Start at top-left corner
  display.println("MOVE B");
  display.setCursor(50,20);             // Start at top-left corner
  display.println(motor2.moved);
  display.display();*/
  motor1.startNewMove();
  motor2.startNewMove();
  
  do{ // run until the handles are at start position
    motor1.run();
    motor2.run();
/*  display.clearDisplay();
  display.setCursor(0,0);             // Start at top-left corner
  display.println("BUG A"); //6
  display.setCursor(50,0);             // Start at top-left corner
  display.println(motor2.debugg);
  display.setCursor(0,10);             // Start at top-left corner
  display.println("POS 1");
  display.setCursor(50,10);             // Start at top-left corner
  display.println(motor2.currentPosition);
  display.setCursor(0,20);             // Start at top-left corner
  display.println("MOVE A"); //-1506
  display.setCursor(50,20);             // Start at top-left corner
  display.println(motor2.moved);
  display.display();*/

  } while(motor1.posReached==false || motor2.posReached==false); // repeat until both handles are at zero position
  PrintDisplayText(0,"HANDLES AT ZERO");
/*  display.clearDisplay();
  display.setCursor(0,0);             // Start at top-left corner
  display.println("BUG A"); //6
  display.setCursor(50,0);             // Start at top-left corner
  display.println(motor2.debugg);
  display.setCursor(0,10);             // Start at top-left corner
  display.println("POS 1");
  display.setCursor(50,10);             // Start at top-left corner
  display.println(motor2.currentPosition);
  display.setCursor(0,20);             // Start at top-left corner
  display.println("MOVE A"); //-1506
  display.setCursor(50,20);             // Start at top-left corner
  display.println(motor2.moved);
  display.display();
  delay(1000);*/
  
}


void newPos(){
  motor1.setDestination(-motor1.currentPosition);
  motor2.setDestination(-motor2.currentPosition);
  motor1.startNewMove();
  motor2.startNewMove();
}
